@@ -13,6 +13,7 @@ from sqlalchemy.orm import sessionmaker
 import uuid
 import json
 import datetime
+from sqlalchemy import select
 
 
 """
@@ -384,7 +385,7 @@ class Kraken_db:
                 obs['value_type'] = record.get('@id', None)
 
 
-        self.session.add(obs)
+        self.session.merge(obs)
 
 
     def _update(self, old_record_type, old_record_id, new_record_type, new_record_id):
@@ -440,7 +441,7 @@ class Kraken_db:
 
         # Step 1. Find DB_observations meeting conditions
         for key, operator, value in params:
-            sub_query = self.session.query(obs0.ref_id, obs0.record_id)
+            sub_query = self.session.query(obs0.ref_id.label('ref_id2'))
 
             if key == 'record_type':
                 sub_query = sub_query.filter(obs0.record_type.like(value))
@@ -456,9 +457,6 @@ class Kraken_db:
                         sub_query = sub_query.filter(obs0.key == key)
                         sub_query = sub_query.filter(obs0.value == value)
 
-
-
-
                 # if number
                 elif isinstance(value, (int, float)):
 
@@ -544,24 +542,67 @@ class Kraken_db:
         if offset:
             query = query.offset(offset)
 
-        query = query.subquery()
-
 
 
         # Step 2. Retrieve all DB_observations with same ref_id
-        #obs1 = aliased(DB_observation)
-        subquery1 = self.session.query(DB_observation)
-        subquery1 = subquery1.join(query, query.c.ref_id == DB_observation.ref_id)
-
-
+        subquery = query
+        subquery = subquery.subquery()
         
+        
+        obs1 = aliased(DB_observation)
+        query = self.session.query(obs1)
+
+        query = query.join(subquery, subquery.c.ref_id2 == obs1.ref_id )
 
 
-        return subquery1
+
+        return query
+
+
+    def test(self):
+        
+        query = None
+
+        obs0 = aliased(DB_observation)
+        sub_query0 = self.session.query(obs0.ref_id.label("ref_id2"))
+
+        sub_query0 = sub_query0.filter(obs0.key == 'schema:qty')
+        sub_query0 = sub_query0.filter(obs0.value > 2)
+        sub_query0 = sub_query0.distinct(obs0.ref_id)
+
+        obs1 = obs0 #aliased(DB_observation)
+        sub_query1 = self.session.query(obs1.ref_id.label("ref_id2"))
+        sub_query1 = sub_query1.filter(obs1.key == 'schema:qty')
+        sub_query1 = sub_query1.filter(obs1.value < 5)
+        sub_query1 = sub_query1.distinct(obs1.ref_id)
+
+
+        query_s = sub_query0.intersect(sub_query1)
+
+
+
+        query_s = query_s.subquery()
 
 
 
 
+        obs_all = aliased(DB_observation)
+
+        query = self.session.query(obs_all)
+
+        for i in query:
+            print(i.value, i.ref_id)
+
+
+        query = query.join(query_s, obs_all.ref_id == query_s.c.ref_id2)
+
+
+
+
+        #print(len(query))
+        for i in query:
+            print(i.value)
+       
 
 
     """
